from __future__ import unicode_literals
from future.standard_library import install_aliases
install_aliases()

from dns.resolver import query, NoAnswer
from tldextract import extract
from urllib.parse import urlparse

from .ip import ip_abuse


def url_abuse(url):
    url = url if '://' in url else "http://" + url
    url = urlparse(url)

    parts = extract(url.netloc.split(':')[0])
    ip_addresses = set()

    if parts.registered_domain:
        answers = query(parts.registered_domain, 'A')
        for rdata in answers:
            ip_addresses.add(rdata.address)
    else:
        ip_addresses.add(parts.domain)

    results = {'value': [], 'names': [], 'abuse': [], 'raw': ""}
    for ip in ip_addresses:
        results['value'].append(ip)
        ip_results = ip_abuse(ip)
        results['raw'] += "IP: {}\n\n{}\n\n".format(ip, ip_results['raw'])
        for key in ['names', 'abuse']:
            for value in ip_results[key]:
                if value not in results[key]:
                    results[key].append(value)

    return results



from __future__ import unicode_literals
from builtins import str

from ipwhois import IPWhois
from operator import itemgetter
from ipaddress import IPv4Network, IPv4Address
import re


def _get_abuse_emails(raw_whois):
    score = 0
    email_candidates = set()

    for line in raw_whois.splitlines():
        email_addresses = re.findall(r'[\w\.+-]+@[\w\.-]+', line)
        if email_addresses:
            abuse_references = line.count('abuse')

            if abuse_references == score:
                email_candidates = set(list(email_candidates) + email_addresses)
            elif abuse_references > score:
                email_candidates = set(email_addresses)
                score = abuse_references

    return list(email_candidates)


def _get_names(ip_address, parsed_whois):
    ip_address = IPv4Address(str(ip_address))
    names = []

    for network in parsed_whois['nets']:
        for cidr in network['cidr'].split(','):
            cidr = IPv4Network(cidr.strip())
            if ip_address in cidr and network['description']:
                names.append([cidr.prefixlen, network['description'].splitlines()[0]])
                break

    return [n[1] for n in sorted(names, key=itemgetter(0), reverse=True)]


def ip_abuse(ip_address):
    obj = IPWhois(ip_address)
    results = obj.lookup_whois(inc_raw=True)

    return {
        "value": ip_address,
        "names": _get_names(ip_address, results),
        "abuse": _get_abuse_emails(results['raw']),
        "raw": results['raw']
    }



from __future__ import unicode_literals

from dns.resolver import query, NoAnswer
from tldextract import extract

from .domain import domain_abuse


def email_abuse(email):
    email_domain = email.split('@')[1]
    results = domain_abuse(email_domain, registrant=True)

    if len(results['abuse']) == 0:
        alternative_domains = set()
        try:
            answers = query(email_domain, 'MX')
        except NoAnswer:
            answers = []

        for rdata in answers:
            parts = extract(str(rdata.exchange))
            if parts.registered_domain != email_domain and parts.registered_domain not in alternative_domains:
                alternative_domains.add(parts.registered_domain)

        for domain in alternative_domains:
            domain_results = domain_abuse(domain, registrant=True)
            results['raw'] += "\n{}\n{}\n".format(domain, domain_results['raw'])
            for key in ['names', 'abuse']:
                for value in domain_results[key]:
                    if value not in results[key]:
                        results[key].append(value)

    return results



from __future__ import print_function,unicode_literals

from pythonwhois.net import get_whois_raw
from pythonwhois.parse import parse_raw_whois
from tldextract import extract


def _get_registrant_abuse(domain, parsed_whois):
    names = []
    emails = []

    if 'contacts' in parsed_whois:
        if 'registrant' in parsed_whois['contacts'] and parsed_whois['contacts']['registrant']:
            if 'organization' in parsed_whois['contacts']['registrant'] and parsed_whois['contacts']['registrant']['organization']:
                names.append(parsed_whois['contacts']['registrant']['organization'])
            elif 'name' in parsed_whois['contacts']['registrant'] and parsed_whois['contacts']['registrant']['name']:
                names.append(parsed_whois['contacts']['registrant']['name'])

        for _, contact in parsed_whois['contacts'].items():
            if contact and 'email' in contact and 'abuse' in contact['email']:
                emails.append(contact['email'])

    return {
        "value": domain,
        "names": names,
        "abuse": emails
    }


def _get_registrar_abuse(domain, parsed_whois):
    names = []
    emails = []

    if 'registrar' in parsed_whois:
        names = parsed_whois['registrar']

    if 'emails' in parsed_whois:
        emails = parsed_whois['emails']

    return {
        "value": domain,
        "names": names,
        "abuse": emails
    }


def domain_abuse(domain, registrant=False):
    parts = extract(domain)
    domain = parts.registered_domain

    try:
        data = get_whois_raw(domain)
        parsed = parse_raw_whois(data, ['Domain', 'contacts'])
    except Exception as e:
        print("Could not get WHOIS for {} ({})".format(domain, e))

        return {
            "value": domain,
            "names": [],
            "abuse": [],
            "raw": ""
        }

    if registrant:
        results = _get_registrant_abuse(domain, parsed)
    else:
        results = _get_registrar_abuse(domain, parsed)

    results['raw'] = "\n\n".join(data)

    return results



# -*- coding: utf-8 -*-
from setuptools import setup

setup(name='abuse_finder',
      version='0.3',
      description='Look for abuse contacts for IP, domain names, email addresses and URLs.',
      url='https://github.com/certsocietegenerale/abuse_finder',
      author='CERT Société Générale',
      author_email='cert.sg@socgen.com',
      license='GPLv3',
      packages=['abuse_finder'],
      install_requires=[
        'ipwhois',
        'ipaddress',
        'pythonwhois-alt>=2.4.6',
        'tldextract',
        'future',
        'dnspython'
      ],
      zip_safe=False)
